//  RandNum.cpp
//  Written by:  Brian Lee Bowers
//  31 July 1998

//-  Include Header Files  -//
#include "RandNum.h"

//==========================================================
//  Function:  PowTwo()
//  Purpose:  return a value near MAX_INT
//  Accepts:  void
//  Returns:  unsigned int
//  Written by:  Brian Lee Bowers
//  Revisions:  0.0.1  31 July 1998
//
//  0.0.1  There is no easy way to get a power of two.
//    This function raises two to the power of
//    (sizeof(unsigned int) * 8) - 1
//==========================================================

unsigned int PowTwo(void)
{
  unsigned int sz = (sizeof(unsigned int) * 8) - 1;
  unsigned int result = 1;
  for (unsigned int i=1; i<sz; i++)
    result *= 2;
  return result;
}

//==========================================================
//  Function:  RandNum::RandNum()
//  Purpose:  constructor
//  Accepts:  unsigned long
//  Returns:  no return type allowed for constructor
//  Written by:  Brian Lee Bowers
//  Revisions:  0.0.1  31 July 1998
//
//  0.0.1  The constructor here simply initializes the
//    values to be used by the class for the rest of its
//    scope.  b must be large, but smaller than m, and
//    end with x21 where x is positive.  m should probably
//    be some value near, but less than, 2^(sizeof(int)*8)
//==========================================================

RandNum::RandNum(unsigned long seed) :
  a(seed),
  b(1163221UL),
  m(PowTwo())
{}

//==========================================================
//  Function:  RandNum::~RandNum()
//  Purpose:  destructor
//  Accepts:  void
//  Returns:  no return type allowed for destructor
//  Written by:  Brian Lee Bowers
//  Revisions:  0.0.1  31 July 1998
//
//  0.0.1  Provided as a stub
//==========================================================

RandNum::~RandNum(void)
{}

//==========================================================
//  Operator:  RandNum::operator()()
//  Purpose:  return a pseudo-random number
//  Accepts:  unsigned int
//  Returns:  unsigned int
//  Written by:  Brian Lee Bowers
//  Revisions:  0.0.1  31 July 1998
//
//  0.0.1  This operator makes use of the complex formula
//    for determining a pseudo-random number using the
//    linear congruential method.  The simplified formula
//    is a(n) = ((a(n-1) * b) + 1) % m.  The complex formula
//    avoids overflow problems inherent in the simplified
//    formula.
//==========================================================

unsigned int RandNum::operator()(unsigned int range)
{
  unsigned int p0 = a%m;
  unsigned int p1 = a/m;
  unsigned int q0 = b%m;
  unsigned int q1 = b/m;
  unsigned long m2 = (unsigned long)m * (unsigned long)m;

  a = (((unsigned long)(p1 * q0) % m2) +
       ((unsigned long)(p0 * q1) % m2) +
       ((unsigned long)(p0 * q0)) + 1) % m;

  return (a % range);
}
